package com.ether.dao;

import com.ether.IsotopeG03;
import com.ether.Pair;
import com.ether.PersistenceException;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.jetbrains.annotations.NotNull;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * @author npipsl
 * @date july 2012
 */
public class IsotopeG03DAOImpl
        extends DomainAccessObjectImpl<IsotopeG03, Integer>
        implements IsotopeG03DAO
{
    protected IsotopeG03DAOImpl()
    {
        super( IsotopeG03.class );
    }

    @NotNull
    public List<IsotopeG03> getAllIsotopeG03()
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeG03.class );
        return selectAllByCriteria( IsotopeG03.class, criteria );
    }

    @NotNull
    public List<IsotopeG03> getTransitionsByIsotopeG03Name( final List<String> isotopeNameArray, final Float spectralRangeLower, final Float spectralRangeUpper )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeG03.class );
        criteria.add( Restrictions.in( "chMoleIsot", isotopeNameArray ) );
        criteria.add( Restrictions.ge( "wavenbMax", spectralRangeLower ) );
        criteria.add( Restrictions.le( "wavenbMin", spectralRangeUpper ) );
        return selectAllByCriteria( IsotopeG03.class, criteria );
    }

    @NotNull
    public Set<String> getTransitionsUpperByTransitionLower( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper , final String transitionLower)
            throws PersistenceException
    {

          final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) )
                .add( Restrictions.eq( "codeQuantLower",transitionLower ) );
        criteria.addOrder( Order.asc( "codeQuantUpper" ) );
        criteria.setProjection( Projections.distinct( Projections.property( "codeQuantUpper" ) ));


        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String>( transitions );
    }

    @NotNull
    public Set<String> getTransitionsUpperByTransitionUpper( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper , final String transitionUpper)
            throws PersistenceException
    {

        final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) )
                .add( Restrictions.eq( "codeQuantUpper",transitionUpper ) );
        /*classer par ordre croissant*/
        criteria.addOrder( Order.asc("codeQuantLower") );
        /*enlever les doublons*/
        criteria.setProjection( Projections.distinct( Projections.property( "codeQuantLower" ) ));


        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String>( transitions );
    }

    @NotNull
    public Set<String> getTransitionsUpper( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper )
            throws PersistenceException
    {
        return getTransitions(isotopesSelectedNameList, spectralRangeLower, spectralRangeUpper, true);
    }

    @NotNull
    public Set<String> getTransitionsLower( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper )
            throws PersistenceException
    {
        return getTransitions(isotopesSelectedNameList, spectralRangeLower, spectralRangeUpper, false);
    }

    @NotNull
    private Set<String> getTransitions( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper, final boolean isForUpper )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) );

        if(isForUpper)
            criteria.addOrder( Order.asc( "codeQuantUpper" ) )
                    .setProjection( Projections.distinct( Projections.property( "codeQuantUpper" ) ) );
        else
            criteria.addOrder( Order.asc("codeQuantLower") )
                    .setProjection( Projections.distinct( Projections.property( "codeQuantLower" ) ) );

        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String> ( transitions );
    }

}