package com.ether.dao;

import com.ether.IsotopeG09;
import com.ether.IsotopeIasiG03;

import com.ether.PersistenceException;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.jetbrains.annotations.NotNull;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
/**
 * @author npipsl
 * @date july 2012
 */


public class IsotopeIasiG03DAOImpl
        extends DomainAccessObjectImpl<IsotopeIasiG03, Integer>
        implements IsotopeIasiG03DAO
{
    protected IsotopeIasiG03DAOImpl()
    {
        super( IsotopeIasiG03.class );
    }

     @NotNull
    public Set<String> getTransitionsUpperByTransitionLower( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper , final String transitionLower)
            throws PersistenceException
    {

          final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeIasiG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) )
                .add( Restrictions.eq( "codeQuantLower",transitionLower ) );
        criteria.addOrder( Order.asc( "codeQuantUpper" ) );
        criteria.setProjection( Projections.distinct( Projections.property( "codeQuantUpper" ) ));

        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String>( transitions );
    }

     @NotNull
    public Set<String> getTransitionsUpperByTransitionUpper( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper , final String transitionUpper)
            throws PersistenceException
    {

          final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeIasiG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) )
                .add( Restrictions.eq( "codeQuantUpper",transitionUpper ) );
        criteria.addOrder( Order.asc("codeQuantLower") );
        criteria.setProjection( Projections.distinct( Projections.property( "codeQuantLower" ) ));

        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String>( transitions );
    }


    @NotNull
    public Set<String> getTransitionsUpper( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper )
            throws PersistenceException
    {
        return getTransitions(isotopesSelectedNameList, spectralRangeLower, spectralRangeUpper, true);
    }

    @NotNull
    public Set<String> getTransitionsLower( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper )
            throws PersistenceException
    {
        return getTransitions(isotopesSelectedNameList, spectralRangeLower, spectralRangeUpper, false);
    }

    @NotNull
    private Set<String> getTransitions( final List<String> isotopesSelectedNameList, final Float spectralRangeLower, final Float spectralRangeUpper, final boolean isForUpper )
            throws PersistenceException
    {
        final DetachedCriteria criteria = DetachedCriteria.forClass( IsotopeIasiG03.class )
                .add( Restrictions.in( "chMoleIsot", isotopesSelectedNameList ) )
                .add( Restrictions.ge( "wavenbMax", spectralRangeLower ) )
                .add( Restrictions.le( "wavenbMin", spectralRangeUpper ) );

        if(isForUpper)
            criteria.addOrder( Order.asc( "codeQuantUpper" ) )
                    .setProjection( Projections.distinct( Projections.property( "codeQuantUpper" ) ) );
        else
            criteria.addOrder( Order.asc("codeQuantLower") )
                    .setProjection( Projections.distinct( Projections.property( "codeQuantLower" ) ) );

        final List<String> transitions = selectAllByCriteria( String.class, criteria );
        return new LinkedHashSet<String> ( transitions );
    }

}